/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
#include "nsISupports.idl"

[scriptable, uuid(c73294b2-b619-4915-b0e8-314d4215e08d)]
interface nsIMsgPurgeService : nsISupports {

  void init();
  void shutdown();
};

