/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


.activityview {
  -moz-appearance: none !important;
  margin: 0;
  border-bottom: 2px solid;
  -moz-border-bottom-colors: ThreeDHighlight ThreeDLightShadow;
}

#search {
  -moz-box-pack: end;
}

#clearListButton {
  min-height: 0;
  min-width: 0;
  margin-top: 3px;
}

/**
 * Images for buttons in the interface
 */

activity-process button {
  list-style-image: url(chrome://messenger/skin/activity/buttons.png);
}
.cancel {
  -moz-image-region: rect(0px, 16px, 16px, 0px);
}
.cancel:hover {
  -moz-image-region: rect(0px, 32px, 16px, 16px);
}
.cancel:hover:active {
  -moz-image-region: rect(0px, 48px, 16px, 32px);
}

.pause {
  -moz-image-region: rect(48px, 16px, 64px, 0px);
}
.pause:hover {
  -moz-image-region: rect(48px, 32px, 64px, 16px);
}
.pause:not([disabled="true"]):hover:active {
  -moz-image-region: rect(48px, 48px, 64px, 32px);
}
.pause[disabled="true"] {
  -moz-image-region: rect(48px, 16px, 64px, 0px);
}

.resume {
  -moz-image-region: rect(16px, 16px, 32px, 0px);
}
.resume:hover {
  -moz-image-region: rect(16px, 32px, 32px, 16px);
}
.resume:hover:active {
  -moz-image-region: rect(16px, 48px, 32px, 32px);
}

.retry {
  -moz-image-region: rect(32px, 16px, 48px, 0px);
  display: none;
}
.retry:hover {
  -moz-image-region: rect(32px, 32px, 48px, 16px);
}
.retry:hover:active {
  -moz-image-region: rect(32px, 48px, 48px, 32px);
}
.retry[enabled="true"] {
  display: inline;
}

.blockedIcon {
  list-style-image: url(chrome://global/skin/icons/Error.png);
}

activity-process button {
  -moz-appearance: none;
  min-height: 16px;
  min-width: 16px;
  max-height: 16px;
  max-width: 16px;
  padding: 0;
  margin: 0 1px;
}

activity-event button {
  list-style-image: url(chrome://messenger/skin/activity/buttons.png);
}
.undo {
  -moz-image-region: rect(32px, 16px, 48px, 0px);
}
.undo:hover {
  -moz-image-region: rect(32px, 32px, 48px, 16px);
}
.undo:hover:active {
  -moz-image-region: rect(32px, 48px, 48px, 32px);
}

activity-event button {
  -moz-appearance: none;
  min-height: 16px;
  min-width: 16px;
  max-height: 16px;
  max-width: 16px;
  padding: 0;
  margin: 0 1px;
}

activity-warning button {
  list-style-image: url(chrome://messenger/skin/activity/buttons.png);
}
.recover {
  -moz-image-region: rect(32px, 16px, 48px, 0px);
}
.recover:hover {
  -moz-image-region: rect(32px, 32px, 48px, 16px);
}
.recover:hover:active {
  -moz-image-region: rect(32px, 48px, 48px, 32px);
}

activity-warning button {
  -moz-appearance: none;
  min-height: 16px;
  min-width: 16px;
  max-height: 16px;
  max-width: 16px;
  padding: 0;
  margin: 0 1px;
}

.activitygroupbox activity-event,
.activitygroupbox activity-warning,
.activitygroupbox activity-process {
  -moz-padding-start: 3px;
}

activity-group button {
  list-style-image: url(chrome://messenger/skin/activity/buttons.png);
}
.retry {
  -moz-image-region: rect(32px, 16px, 48px, 0px);
}
.retry:hover {
  -moz-image-region: rect(32px, 32px, 48px, 16px);
}
.retry:hover:active {
  -moz-image-region: rect(32px, 48px, 48px, 32px);
}

activity-group button {
  -moz-appearance: none;
  min-height: 16px;
  min-width: 16px;
  max-height: 16px;
  max-width: 16px;
  padding: 0;
  margin: 0 1px;
}

/* Specialized Activity Bindings */

activity-process {
  padding: 3px 8px;
  min-height: 40px !important;
}

activity-process .statusText {
  font-size: smaller;
  color: #555;
}

activity-event .dateTime,
activity-event .statusText {
  font-size: smaller;
  color: #555;
}

activity-warning .dateTime,
activity-warning .statusText {
  font-size: smaller;
  color: #555;
}


.processIconBox image {
  list-style-image: url(chrome://messenger/skin/activity/defaultProcessIcon.png);
}

.eventIconBox image {
  list-style-image: url(chrome://messenger/skin/activity/defaultEventIcon.png);
}

.warningIconBox image {
  list-style-image: url(chrome://messenger/skin/activity/defaultWarningIcon.png);
}

image.syncMail {
  list-style-image: url(chrome://messenger/skin/activity/syncMailIcon.png);
}

image.sendMail {
  list-style-image: url(chrome://messenger/skin/activity/sendMailIcon.png);
}

image.copyMail {
  list-style-image: url(chrome://messenger/skin/activity/copyMailIcon.png);
}

image.moveMail {
  list-style-image: url(chrome://messenger/skin/activity/moveMailIcon.png);
}

image.deleteMail {
  list-style-image: url(chrome://messenger/skin/activity/deleteMailIcon.png);
}

image.addItem {
  list-style-image: url(chrome://messenger/skin/activity/addItemIcon.png);
}

image.removeItem {
  list-style-image: url(chrome://messenger/skin/activity/removeItemIcon.png);
}

image.undo {
  list-style-image: url(chrome://messenger/skin/activity/undoIcon.png);
}

image.indexMail {
  list-style-image: url(chrome://messenger/skin/activity/indexMailIcon.png);
}

image.compactMail {
  list-style-image: url(chrome://messenger/skin/activity/compactMailIcon.png);
}

/* prevent flickering when changing states */
.activityTypeIcon {
  min-height: 32px;
  min-width: 32px;
  -moz-padding-end: 2px;
}

.activityitem:not([selected="true"]):nth-child(odd) {
  background-color: -moz-oddtreerow;
}

/* these are copied from richlistbox.css, to give us a starting point on selection */

.activityitem {
  padding: 7px 4px;
  background-color: -moz-Field;
  color: -moz-FieldText;
  border-bottom: 1px solid #f1efe2;
}

.activityitem[selected="true"] {
  background-color: -moz-Dialog;
  color: -moz-DialogText;
}

richlistbox:focus .activityitem[selected="true"],
richlistbox:focus activity-group[selected="true"] {
  background-color: Highlight;
  color: HighlightText;
}

richlistbox:focus .activityitem[selected="true"] .statusText,
richlistbox:focus .activityitem[selected="true"] .dateTime,
richlistbox:focus .activityitem[selected="true"] .displayText,
richlistbox:focus .activityitem[selected="true"] .completionTime {
  color: HighlightText;
}
